
;****************************************************************************
;* Filename: PROBE.ASM
;****************************************************************************
;* Author:   Voja Antonic
;* Company:  PC Press
;* Revision: RevA0
;* Date:     6-29-98
;* Assembled using MPASM rev 01.50
;****************************************************************************
;* Include files:
;*      p16f84.inc
;****************************************************************************
;*
;* This is the program for multi-purpose laboratory instrument,
;*  consisting of logic probe, single-channel logic state analyzer,
;*  serial code receiver and frequency counter. As this is single-chip
;*  instrument, all functions are supported by software.
;*  
;* LCD module used is Hitachi's LM032L with 2 lines of 20 columns.
;*  
;* Note: The code is optimized for code space, and for that reason the
;*  most of code could not be written in modular format. For the same
;*  reason a lot of subroutines have more than one entry point and some
;*  of them are terminated by GOTO instead of RETURN.
;*
;* I/O port usage (all PORTA bits are inputs, all PORTB bits outputs)
;*  (note: bits 0, 5, 6 and 7 in port B have two functions each):
;*  
;*  PORTA,0 (input)   probe input
;*  PORTA,1 (input)   voltage monitor (high if battery voltage < 4 V)
;*  PORTA,2 (input)   left key (key 1) (low = key pressed)
;*  PORTA,3 (input)   right key (key 2) (low = key pressed)  
;*  PORTA,4 (input)   probe input
;* 
;*  PORTB,0 (output)  eneble LCD (high = select LCD), discharge (high = on)
;*  PORTB,1 (output)  register select LCD (low = instruction, high = data)
;*  PORTB,2 (output)  hi-imp output for probe pin
;*  PORTB,3 (output)  current hold for MCU supply (low = off)
;*  PORTB,4 (output)  LCD module D4
;*  PORTB,5 (output)  LCD module D5, LED P (high = on)
;*  PORTB,6 (output)  LCD module D6, LED H (high = on)
;*  PORTB,7 (output)  LCD module D7, LED L (high = on)
;*
;*
;* External Clock Frequency: 10 MHz
;* Config Bit Settings: CP=OFF, PWRTE=ON, WDT=OFF, OSC=HS 
;* Program Memory Usage: 1024 words
;* Data EEPROM Usage: 61 bytes (Written by programmer and used as
;*               read-only, contents will not be modified during operation)
;* Data RAM Usage: 68 bytes
;*
;****************************************************************************

	list p=16f84, f=inhx8m, n=0
	include "p16f84.inc"

FLAG	equ	0ch	;  1 by  flag register
RXBITS	equ	0dh	;  1 by  bit0=parity, bit 1=7/8 bits, bit 2=inverse
DJNZ	equ	0eh	;  1 by  general purpose, e.g. loop counter
SCRATCH	equ	0fh	;  1 by  general purpose scratchpad
PCOUNT	equ	10h	;  1 by  timing count for led P (monostable simulator)
SUBMODE	equ	11h	;  1 by  submode (cursor horizontal position)
DEBO1	equ	12h	;  1 by  rotor for key 1 debouncing
DEBO2	equ	13h	;  1 by  rotor for key 2 debouncing
COUNT	equ	14h	;  1 by  general purpose counter
RATE	equ	15h	;  1 by  analyzer sample rate, 0...15
CHARCOU	equ	16h	;  1 by  char counter for fixed format display
SHOWCOU	equ	17h	;  1 by  1-4, which group of 60 samples is displayed
DELAYL	equ	18h	;  1 by  delay for led P on when led L is on
DELAYH	equ	19h	;  1 by  delay for led P on when led H is on
PRESC	equ	1ah	;  1 by  prescaler rate for frequency counter
TIMOUTL	equ	1bh	;  1 by  timeout counter lo, for auto power off
TIMOUTH	equ	1ch	;  1 by  timeout counter hi, for auto power off
RXRATE	equ	1dh	;  1 by  rx baud rate, 0...7
BIN4	equ	1eh	;  4 by  arith buf bin value, 4 by, lo byte first
CMP4	equ	22h	;  4 by  arith buf for comparing, 4 by, lo byte first
BUFFER	equ	26h	; 42 by  42 by receive buffer for analyzer and RX

REL	equ	1	; =1 to place cursor on 1st char of command (default)
			; =0 to place cursor before the command

; Bits definitions for FLAG register (bit 0 not used):

DP	equ	1	; decimal point in 3-digit bin2dec conv
PTIP	equ	2	; prev.state of probe input (for edge detect)
RIPPLE	equ	3	; zero blanking bit
XTOX	equ	4	; analyzer start at: 1=rising, 0=falling edge
LEDP	equ	5	; led Pulse, 1=on
LEDH	equ	6	; led High, 1=on
LEDL	equ	7	; led Low, 1=on


;*****************************************************************************
;* Reset vector
;*****************************************************************************

	goto	Start

;*****************************************************************************
;* Get1MHz
;* This subroutine fetches 307 samples (last 7 will be ignored) from PORTA.0
;*  rotating throughu CARRY at 1 MHz rate - 2.5 instruction cycles for each
;*  sample, realized mostly as as 2 and 3 cycles alternatively, at the
;*  following order:
;*  
;*  4t-2t-2t (not in main loop, executed only once), and then
;*  
;*  2t-2t-3t-2t-3t-2t-3t-2t-3t-2t-3t-2t-3t-2t-2t-4t    (repeated 19 times)
;*  
;* Call Common initializes loop counter (COUNT) to make 19 cycles before
;*  exiting (16 samples are fetched at each pass), and FSR to point to
;*  BUFFER. It also presets T0SE bit depended on XTOX bit (in FLAG register)
;*  to enable proper edge detecting, as it will affect TMR0 state.
;*  State of key 2 (Break) is tested while waiting for starting condition.
;* Write pointer FSR is incremented after every 8 samples. COUNT initial
;*  value is 01101101 and, after ANDing 0c0h and subtracting 33h from it,
;*  makes 0dh, even if COUNT is incremented 18 times. After 19 passes,
;*  COUNT is incremented to b'10000000', which after AND 0c0h and
;*  SUB 33h makes 4dh. Those jumps are location sensitive, and it makes the
;*  whole subroutine unrelocateable.
;* Between this subroutine and the instruction goto Finished (below), which
;*  must be at location 4dh, there are 25 free locations. They are used for
;*  tables DecTab and CurTab, which causes that those tables must have the
;*  fixed length. If anything is relocated here, take care not to affect
;*  location of instruction goto Finished.
;*
;* Input variables: None
;* Output variables: None
;*****************************************************************************

  	org	1		; this subroutine must start at address 1

Get1MHz				; 2.5 t read cycle
  	movlw	80h-.19		; loop end after 19 cycles (38 by=304 samples)
	call	Common
GetEdge
	btfss	PORTA,3		; test for Break key
	goto	Break

	movf	TMR0,W
	rrf	PORTA,F	 ; <---	; the first sample is a little earlier
				; ... to compensate starting delay
	btfsc	STATUS,Z	; worst case delay: 4 read cycles
	goto	GetEdge		; wait for event

	rrf	INDF,F
	rrf	PORTA,F	 ; <---
	rrf	INDF,F
	rrf	PORTA,F	 ; <---
				; movwf PCL will jump here at 18 passes
				; ...@addr 0100 0000 (40h) - 33h = 0dh
	rrf	INDF,F
	rrf	PORTA,F	 ; <---
	rrf	INDF,F
	rrf	PORTA,F	 ; <---
	rrf	INDF,F
	rrf	PORTA,F	 ; <---
	rrf	INDF,F

	incf	COUNT,F		; COUNT = loop counter

	rrf	PORTA,F	 ; <---
	rrf	INDF,F
	rrf	PORTA,F	 ; <---
	rrf	INDF,F

	incf	FSR,F		; must be exactly 8 read cycles distance
				; ... between FSR incrementing
	rrf	PORTA,F	 ; <---
	rrf	INDF,F
	rrf	PORTA,F	 ; <---
	rrf	INDF,F

	movf	COUNT,W		; COUNT = loop counter

	rrf	PORTA,F	 ; <---
	rrf	INDF,F
	rrf	PORTA,F	 ; <---
	rrf	INDF,F

	andlw	0c0h		; this will make first 18 jumps to 0dh,
				; ...and the 19th one to 4dh
	rrf	PORTA,F	 ; <---
	rrf	INDF,F
	rrf	PORTA,F	 ; <---
	rrf	INDF,F

	addlw	-33h		; this will make first 18 jumps to 0dh,
				; ...and the 19th one to 4dh
	rrf	PORTA,F	 ; <---
	rrf	INDF,F
	rrf	PORTA,F	 ; <---
	rrf	INDF,F

	incf	FSR,F		; must be exactly 8 read cycles distance
				; ... between FSR incrementing
	rrf	PORTA,F	 ; <---
	rrf	INDF,F
	rrf	PORTA,F	 ; <---
	rrf	INDF,F
	rrf	PORTA,F	 ; <---	; worst case jitter: 0.4 read cycles
 
	movwf	PCL		; jumps on 0dh in first 18 passes
				; jumps on 4dh at 19th pass

;*****************************************************************************
;* This table is used for bin2ascii (4-byte to 8-digit) conversion
;*****************************************************************************

DecTab
	dt	098h,096h,080h	; decimal 10 000 000
	dt	00fh,042h,040h	; decimal 1 000 000
	dt	001h,086h,0a0h	; decimal 100 000
	dt	000h,027h,010h	; decimal 10 000
	dt	000h,003h,0e8h	; decimal 1 000
	dt	000h,000h,064h	; decimal 100
	dt	000h,000h,00ah	; decimal 10

;*****************************************************************************
;* Cursor position table for all SUBMODEs in mode 4 (battery manager)
;*****************************************************************************

CurTab4
	dt	0d2h+REL,0c0h,0c3h+REL,0cah+REL

;*****************************************************************************
;* This is exit point for subroutine Get1MHz. Do not move this
;*  instruction, it must be at address 4dh!
;*****************************************************************************

	org	80h-33h		; addr 1000 0000 (80h) - 33h = 4dh
				; Get1MHz jumps here
	goto	Finished

;*****************************************************************************
;* Table for LCD module character generator redefinition, to enable
;*  pseudographic representation of bit samples in analyzer mode. It defines
;*  first 8 characters, which are stored in LCD module's RAM.
;*****************************************************************************

Graphs
	;	...  ..*  .*.  .**  *..  *.*  **.  ***
	dt	00h, 01h, 04h, 05h, 10h, 11h, 14h, 15h

;*****************************************************************************
;* Cursor position table for all SUBMODEs in mode 1 (Analyzer)
;*****************************************************************************

CurTab1
	dt	0d2h+REL,0c0h,0cch+REL,0ceh+REL,0d0h+REL

;*****************************************************************************
;* Cursor position table for all SUBMODEs in mode 2 (Serial rcvr)
;*****************************************************************************

CurTab2
	dt	0d2h+REL,0c7h+REL,0cch+REL,0ceh+REL,0d0h+REL

;*****************************************************************************
;* Prescaler table for resolution display in mode 3 (freq counter)
;*****************************************************************************

PrescTab
	dt	.4, .8, .16, .32

;*****************************************************************************
;* Range table for max. frequency display in mode 3 (freq counter)
;*****************************************************************************

RangeTab
	dt	.5, .10, .20, .40
;*****************************************************************************
;* Timing constants for serial code receiver
;*****************************************************************************

BaudRate
	dt	.188, .94, .46, .23, .11, .5, .3, .1

;*****************************************************************************
;* Text strings (terminator = last character with bit 7 set)
;*****************************************************************************

TxtHz
	dt	"MHz",'/'+80h
DisTxt
	dt	"Off Disch. Charg",'e'+80h
Head1
	dt	"Analyze",'r'+80h
Head2
	dt	"Seria",'l'+80h
Head3
	dt	"Frequenc",'y'+80h
Head4
	dt	"Batter",'y'+80h
BrkMes
	dt	"Brea",'k'+80h

;;*******
;*******  START
;*******

;*****************************************************************************
;* Power Up sequence: I/O port B defined as all outputs, PORTB,3 set to
;*  switch power supply on and the internal Data Ram is cleared
;*****************************************************************************

Start
	bsf	STATUS,RP0
	clrf	TRISB		; port b: all bits outputs, port a: inputs
	bcf	STATUS,RP0
	movlw	0ch
	movwf	PORTB		; switch power supply ON (set PORTB,3)
	call	ClrRam		; clear internal RAM and wait 33.8 ms

;*****************************************************************************
;* LCD module initialization. 4-bit mode selected, display data RAM cleared,
;*  cursor set to blink mode, and the pseudographics character for
;*  character set 00h-07h preset from table Graphs.
;*****************************************************************************

	bcf	PORTB,1		; rs lo (instruction)
	movlw	2		; 4-bit mode
	call	Nibble
	movlw	28h		; function set: 4 bit mode, 2 lines, 5*7 dots
	call	WrComL
	movlw	06h		; mode set: cursor direction right, no shift
	call	WrComL
	movlw	0dh		; display on, no cursor, blink cursor pos
	call	WrComL

	movlw	40h		; cg ram addr 0
	call	WrComL
	movlw	Graphs		; inici graph set for lcd disp
	movwf	SCRATCH
	movlw	.8*.4
	movwf	CHARCOU		; 8 special characters to define
GoGraph
	movlw	.5
	movwf	COUNT
FiveRows
	call	PclSub1
	call	CharNCC		; rows 1-5 from the table
	decfsz	COUNT,F
	goto	FiveRows
	movlw	15h
	call	CharBl		; row 6: all dots set, row 7: all dots cleared
	call	Blank		; row 8: all dots cleared
	incf	SCRATCH,F	; inc ptr
	decfsz	CHARCOU,F
	goto	GoGraph		; loop 8 x

;;*******
;*******  USER INTERFACE
;*******

;*****************************************************************************
;* This is home point for mode 1 (Analyzer): prints text "Analyzer" and
;*  command line in line 2, with cursor placed depended on variable SUBMODE.
;*  Then the keyboard routine is called, where it will wait for key to be
;*  pressed.
;* Break entry point prints message Break in line 1 and readraws line 2
;*****************************************************************************

Mode1				; Mode 1: Analyzer
 	movlw	Head1-1
	call	Headline	; print "Analyzer"
	goto	Farm1
Break				; Break entry pt (if Break pressed in Mode 1)
	call	PrintBrk	; print "Break"
Farm1
	call	PrintM1		; print string in line 2
Farm1B
	movlw	CurTab1
	call	CurPosKb	; place cursor on proper position
				; test keys and probe input, service leds
				; return if key pressed (C:key1, NC:key2)

;*****************************************************************************
;* If key 1 pressed (C), SUBMODE is advanced (range 0...4, then wrap to 0
;* If key 2 pressed (NC), program vectors to corresponding routine
;*  (except if SUBMODE=1, then the sample rate is advanced and displayed)
;*****************************************************************************

	btfsc	STATUS,C
	goto	Key1A
				; key 2 pressed
	call	Range5		; advance variable SUBMODE in range 0...4
	goto	Farm1B
Key1A				; key 1 pressed
	decfsz	SUBMODE,W
	goto	NoRate		; if SUBMODE <>1
				; continue if SUBMODE=1
	incf	RATE,F
	bcf	RATE,4		; RATE range 0...15

	call	ClrRow1		; prepare line 1 to print sample rate No.
	incf	RATE,W		; readjust RATE from 0...15 to 1...16
	call	Print255	; print serial number of sample rate 1...16
	goto	Farm1

EdgeSet				; Change start condition (L-to-H or H-to-L)
	movlw	10h		; bit 4 is flag XTOX
	xorwf	FLAG,F
	goto	Farm1

NoRate
	btfsc	SUBMODE,0
	goto	Mode1Go		; if SUBMODE=3
	btfsc	SUBMODE,1
	goto	EdgeSet		; if SUBMODE=2
	btfsc	SUBMODE,2
	goto	Mode1Show	; if SUBMODE=4
				; if SUBMODE=0 then program drops to Mode2...

;*****************************************************************************
;* This is home point for mode 2 (RS232 receiver): prints text "Serial" and
;*  command line in line 2, with cursor placed depended on variable SUBMODE.
;* BrkRS entry point prints message Break in line 1 and readraws line 2
;*  if no bytes are received, and displays first 7 bytes if any byte received
;*****************************************************************************

Mode2				; mode 2: Serial receiver
	movlw	Head2-1
	call	Headline	; print "Serial"
	goto	Farm2
BrkRS				; Break entry pt (if Break pressed in mode 2)
	movf	FSR,W
	xorlw	BUFFER
	btfss	STATUS,Z
	goto	Show2		; show if at least 1 byte rcvd...
	call	PrintBrk	; print Break

;*****************************************************************************
;* Prints baud rate in KBaud on LCD
;* 
;* Input variables: RXRATE in range 0...7
;* Output variables:
;*     CHARCOU is decremented by the number of characters printed
;*****************************************************************************

Farm2
	movlw	0c8h		; baud rate position on LCD
	call	WrComL		; move cursor command

	bcf	FLAG,DP		; no decimal point printing if RATE=0

	clrf	BIN4+1		; BIN4+1 is high byte for baud rate display

	incf	RXRATE,W
	movwf	DJNZ		; DJNZ = RXRATE+1

	movlw	.115
	btfsc	DJNZ,3
	goto	Lth256		; case 115.2 (RXRATE=7)

	bsf	FLAG,DP		; for rete 0...6 there is a decimal point
	bsf	BIN4+1,1		; BIN4+1 = 2
	movlw	.576-.512
	incf	DJNZ,F
	btfsc	DJNZ,3
	goto	Lth256		; case 57.6 (RXRATE=1)

	clrf	BIN4+1
	movlw	.3
	movwf	BIN4		; BIN4 will be rotated (multiplied by 2)
				; RXRATE+2 times to get 1.2 - 2.4 - 4.8
				; - 9.6 - 19.2 - 38.4
X2Loop
	bcf	STATUS,C
	rlf	BIN4,F
	rlf	BIN4+1,F	; 16-bit rotate
	decfsz	DJNZ,F
	goto	X2Loop
	movf	BIN4,W
Lth256
	call	PrintBR		; print baud rate, including decimal point
	call	Blank		; to delete last numeric from previous rate

;*****************************************************************************
;* Prints number of bits to be received (7 or 8), with suffix "p" if parity
;*  bit will be received (not written to RAM!), and with prefix "i" if
;*  inverse input polarity is expected. Input variable RXBITS, bit 0 set if
;*  parity bit expected, bit 1 set if 8-bit word and bit 2 set if inverse
;*  polarity (low start bit, inverse data bits and high stop bit)
;*****************************************************************************

	movlw	0cch		; pos for bit No. (7/8/7p/8p/i7/i8/i7p/i8p) -1
	call	WrComL		; write command

	movlw	' '
	btfsc	RXBITS,2
	movlw	'i'
	call	Char		; print blank or "i"

	movlw	'7'
	btfss	RXBITS,1
	movlw	'8'
	call	Char		; print "7" or "8" (bits)

	movlw	' '
	btfsc	RXBITS,0
	movlw	'p'
	call	Char		; print "p" (parity bit present) or blank

;*****************************************************************************
;* This call prints number of group displayed and "*" (execution) symbol
;*****************************************************************************

	call	KaoAna		; print rest of line - it is the same as
				; on mode 1 (analyzer)

;*****************************************************************************
;* Places cursor on proper position (input variable SUBMODE) and calls
;*  keyboard subroutine, where it will wait for key to be pressed
;*****************************************************************************

	movlw	CurTab2		
	call	CurPosKb	; place cursor on proper pos
				; test keys and probe input, service leds
				; return if key pressed (C:key1, NC:key2)

;*****************************************************************************
;* If key 1 pressed (C), SUBMODE is advanced (range 0...4, then wrap to 0
;* If key 2 pressed (NC), program vectors to corresponding routine
;*  (except if SUBMODE=1, then the Baud rate is advanced and displayed)
;*****************************************************************************

	btfsc	STATUS,C
	goto	Key1B
				; key 2 pressed
	call	Range5
	goto	Farm2
Key1B				; key 1 pressed
	btfsc	SUBMODE,2
	goto	Mode2Show	; if SUBMODE = 4

	btfss	SUBMODE,1
	goto	Sub01		; if SUBMODE = 0 or SUBMODE = 1

	btfsc	SUBMODE,0
	goto	Mode2Go		; jump if SUBMODE = 3
				; drops here if SUBMODE = 2
	incf	RXBITS,F
	bcf	RXBITS,3	; RXBITS range = 0...7
	goto	Farm2
Sub01
	btfss	SUBMODE,0
	goto	FreqEp		; if SUBMODE = 0 then goto freq counter entry pt

	incf	RXRATE,F	; if SUBMODE = 1 then advance RXRATE
	bcf	RXRATE,3	; RXRATE range 0...7
	goto	Farm2

;*****************************************************************************
;* This subroutine increments variable SUBMODE, and if the result is >4 it
;*  wraps to 0
;*****************************************************************************

Range5				; increment SUBMODE in range 0...4
	incf	SUBMODE,F
	btfsc	SUBMODE,2
	btfss	SUBMODE,0
	return
	clrf	SUBMODE		; SUBMODE cycle in range 0...4
	return

;*****************************************************************************
;* Mode4 is home point for mode 4 (off/discharge/charge): prints text
;*  "Battery" and command line in line 2, with cursor placed depended on
;*  variable SUBMODE. Then the keyboard routine is called, where it will
;*  wait for key to be pressed
;* Break4 entry point prints message Break in line 1 and readraws line 2
;* ExitDis is the entry point if key 2 is pressed during discharging
;*****************************************************************************

ExitDis				; exit discharge entry point (if disch Break)
	call	DisEna30	; switch off PORTB,0 for discharge transistor
Break4				; exit Charge entry point (if Charge Break)
	call	PrintBrk	; print Break
	goto	Contm4
Mode4				; mode 4: discharge/charge
	call	Headline2	; print Battery
Contm4
	call	Row2		; move cursor to line 2
	movlw	DisTxt-1
	call	Write		; print Off Disch Charge
Farm4
	movlw	CurTab4
	call	CurPosKb	; place cursor @ Off/Disch/Charge/-->
				; test keys and probe input, service leds
				; return if key pressed (C:key1, NC:key2)

;*****************************************************************************
;* If key 1 pressed (C), SUBMODE is advanced (range 0...3, then wrap to 0
;* If key 2 pressed (NC), program jumps to corresponding routine
;*****************************************************************************

	btfsc	STATUS,C
	goto	Key1D
				; key 2 pressed
	incf	SUBMODE,F
	bcf	SUBMODE,2	; SUBMODE range 0...3
	goto	Farm4
Key1D				; key 1 pressed
	btfsc	SUBMODE,1
	goto	ChargDis	; goto Charge or discharge process
				; depended on bit 0 in variable SUBMODE
	btfss	SUBMODE,0
	goto	Mode1		; mode wrapped from 4 to 1

	goto	Suicide		; manual power off
;;*******
;*******  CHARGE/DISCHARGE
;*******

;*****************************************************************************
;* Charge/Discharge entry point. If bit SUBMODE,0 set, then go to Charge
;*****************************************************************************

ChargDis
	call	ClrRow1
	btfsc	SUBMODE,0
	goto	Charge		; goto Charge (SUBMODE=3)

;*****************************************************************************
;* Discharge starts here (SUBMODE=2)
;* Cursor moved to line 1 under text "Disch."
;*  Then command 02h (Home Cursor) is issued to LCD controller, but this is
;*  dummy command - the sense is to freeze it after first nibble, and thus to
;*  leave PORTB,0 (ENA) in high state as long as discharging lasts. After the
;*  discharging termination (if voltage monitor detects <4V or key 2 pressed),
;*  the command for LCD controller will be completed, switching
;*  discharging transistor off.
;*  If discharging is broken by key, program returns to user interface for
;*  mode 4, if terminated by voltage monitor, charging takes place
;*****************************************************************************

	bcf	PORTB,1
	clrw
	call	Hinib_B
	call	EnaLCD
	movlw	20h		; LCD command 02h = home cursor
				; this command won't be finished until Break
	call	Hinib_B
	bsf	PORTB,0		; ENA activated
DisLoop
	btfss	PORTA,3
	goto	ExitDis		; discharging manually broken by key

	btfss	PORTA,1		; test voltage monitor
	goto	DisLoop
				; discharging over (voltage < 4V)

	call	DisEna30	; switch off discahge transistor

;*****************************************************************************
;* Charging starts here (by command or after successful discharging)
;* Minute and hour counters are initialized and counting process starts.
;* Clock (in format HH:MM) is displayed in line 1 under text "Charge".
;* If charging broken by key, program returns to user interface for
;*  mode 4, if terminated by timeout (14h), the unit jumps to SUICIDE
;*  (switches off the unit forcing the output PORTB,3 low).
;*****************************************************************************

Charge				; Charge entry point

	clrf	TIMOUTL		; minute counter 0...59
	clrf	TIMOUTH		; hour counter 0...13
ChLoop
	movlw	8bh
	call	WrComL		; cursor digital clock
	call	PrintTL		; print hour in format HH
	movlw	':'
	call	ChPtl		; print minute in format MM

	movlw	.228		; 228 x 263270.4 us = 60s
	movwf	SCRATCH
Min1
	call	GoLoop		; 1283t (513.2us) inclusive	; 1283t
	call	GoLoop		; total 1026.4us		; 1283t

	btfss	PORTA,3		; 2				; 2
	goto	Break4		;  charging manually broken by key

	decfsz	COUNT,F		; 1				; 1
	goto	Min1		; 2  inner pass 1028.4 us	; 2

	decfsz	SCRATCH,F
	goto	Min1		;    pass 263270.4 us

	incf	TIMOUTL,F	; TIMOUTL = minute up counter
	movf	TIMOUTL,W
	addlw	-.60		; test if 60 minutes of charging
	btfss	STATUS,C
	goto	NotHour		; not yet hour advance
	clrf	TIMOUTL
	incf	TIMOUTH,F	; TIMOUTH = hour up counter
NotHour
	movf	TIMOUTH,W
	addlw	-.14		; test if 14 hours of charging
	btfss	STATUS,C
	goto	ChLoop
	goto	Suicide		; charging auto termination after 14 h

;;*******
;*******  KEYBOARD AND PROBE
;*******

;*****************************************************************************
;* CurPosKb
;* This subroutine places cursor in line 2 at position taken from the
;*  lookup table: table offset is addressed by W at input, and the table read
;*  location by variable SUBMODE.
;* High timeout counter (TIMOUTH) is initialized. This sets automatic
;*  Power Off timing to about 8 minutes. TIMOUTL is of minor importance
;*  here (it affects less than 2 seconds of timing), so it was not worth
;*  waisting one instruction word.
;* Bit DEBO,0 is set to disable false recognizing of PORTA,3 low level as
;*  falling edge (as if key 2 was just pressed). This could happen if some
;*  function was broken by pressing key 2, as those are simple port tests
;*  without affecting debouncer.
;* This subroutine continues to keyboard scan subroutine.
;*
;* Input variables: SUBMODE, affects cursor position
;* Output variables: TIMOUTH=0, high timeout counter
;*****************************************************************************

CurPosKb
	addwf	SUBMODE,W
	call	PclSub
	call	WrComL
	clrf	TIMOUTH

	bsf	DEBO1,0
	bsf	DEBO2,0		; to disable false recognizing of low
				; level as falling edge

;*****************************************************************************
;* GoKbd
;* This is the main keyboard subroutine, in which program loops all the time
;*  except in frequency counter mode, or while waiting for start condition or
;*  executing some command. Program exits this subroutine only if some key is
;*  just pressed (not if continuosly pressed), or when timeout counter
;*  (TIMOUTH, TIMOUTL) after 8 min reaches zero. If key 1 was pressed, flag
;*  STATUS,C will be reset at exit, if key 2 was pressed, flag STATUS,C will
;*  be set. If timeout detected, the routine SUICIDE is executed (the unit
;*  switched off forcing the output PORTB,3 low). 
;* During keyboard scan, LEDs L, H and P are serviced. Logic state at
;*  PORTA,4 affects LEDs L and H directly, and LED P is under control of down
;*  counter PCOUNT. This counter is initialized at every logic level
;*  transition at PORTA,4, and while counting down, if PCOUNT>0, LED P is
;*  switched on.
;* Loop labeled "Unstable" adds the extra delay which timing is not
;*  constant, but changes from 3 to 49t. This minimizes the interference
;*  between the input scan and tested signal frequency.
;* Counter TMR0 is used for detecting of short pulses. At each
;*  transition detected, TMR0 is cleared, and then periodicaly tested if
;*  counter state was incremented. If so, PCOUNT is initialized and LED P
;*  turned on.
;* Register DJNZ is used as a freerunning counter, which divides loop count
;*  by 256 and slows down PCOUNT countdown and keys scanning. Keys are
;*  debounced and falling edge (pressing moment) detected by rotating
;*  registers DEBO1 and DEBO2, and testing them if the key was unpressed at
;*  least at 7 passes and then pressed at 1 pass.
;*  
;* Input variables: none
;* Output variables: Bit STATUS,C reset if key 1 was pressed, set if key 2
;*****************************************************************************

GoKbd
	movf	DJNZ,W		; 1t to avoid interference, total avg 29t
	iorlw	0f0h		; 1t
	movwf	SCRATCH		; 1t SCRATCH varies from 0f0h to 0ffh
UnStable				;
	incfsz	SCRATCH,F	; 1-17t (average 9)
	goto	UnStable	; 2-32t (average 17)

	movf	TMR0,W
	btfsc	STATUS,Z	; test if there was transition at PORTA,4
	goto	NoIniP
	bsf	PCOUNT,3	; initialize PCOUNT for LED P flashing
	clrf	TMR0		; clear counter
NoIniP
	movlw	28h		; lo2hi transition
	btfsc	PORTA,4
	movlw	38h		; hi2lo transition

	bsf	STATUS,RP0
	movwf	OPTION_REG
	bcf	STATUS,RP0

	bcf	DELAYH,0
	bcf	DELAYL,0
	movlw	4
	xorwf	PORTB,F		; changes state of PORTB.2
	btfsc	PORTA,4
	goto	InputHi
				; ------------ input low
	btfss	PORTB,2
	bsf	DELAYL,0	; switches on led L because PORTB,2 was high
	bcf	FLAG,PTIP
	goto	ContInpX
InputHi				; ------------ input high
	btfsc	PORTB,2
	bsf	DELAYH,0	; switches on led L because PORTB,2 was low
	bsf	FLAG,PTIP
ContInpX				; ----------- moves LHP leds from FLAG to PORTB
	bcf	FLAG,LEDL	; delay for led L
	movf	DELAYL,F
	btfss	STATUS,Z
	bsf	FLAG,LEDL
	rlf	DELAYL,F

	bcf	FLAG,LEDH	; delay for led H
	movf	DELAYH,F
	btfss	STATUS,Z
	bsf	FLAG,LEDH
	rlf	DELAYH,F

	call	MoveLESd	; 11t
				; ----------- ea 256. pass
	incfsz	DJNZ,F
	goto	GoKbd

;------------------------------ passes here at each 256. loop (about 8.9 ms)

	bcf	FLAG,LEDP
	movf	PCOUNT,F
	btfsc	STATUS,Z
	goto	PCount0

	decf	PCOUNT,F

	movf	DELAYL,W
	iorwf	DELAYH,W
	btfss	STATUS,Z

	bsf	FLAG,LEDP	; if PCOUNT>0 then dec PCOUNT, P = L OR H
PCount0
				; ----------- key 2 test (right key)
	swapf	PORTA,W
	movwf	SCRATCH		; SCRATCH,7=key2, SCARTCH,6=key1
	comf	SCRATCH,F
	rlf	SCRATCH,F	; c set if key 2 pressed

	rlf	DEBO2,F
	bcf	STATUS,C	; reset C to notify key 2 pressed
	decf	DEBO2,W		; DEBO2 = b'00000001' if just pressed
	btfsc	STATUS,Z
	return			; ----------- exit 1: key 2 just pressed (NC)

				; ----------- key 1 test (left key)

	rlf	SCRATCH,F	; c set if key 1 pressed

	rlf	DEBO1,F
	bsf	STATUS,C	; set C to notify key 2 pressed
	decf	DEBO1,W		; DEBO1 = b'00000001' if just pressed
	btfsc	STATUS,Z
	return			; ----------- exit 2: key 1 just pressed (C)

	decfsz	TIMOUTL,F	; timeout lo counter
	goto	GoKbd

	decfsz	TIMOUTH,F	; timeout hi counter
	goto	GoKbd
				; ----------- exit 3: timeout

;*****************************************************************************
;* Power Off entry point
;* Wait until both keys off for 34 ms, and then switch power off.
;* PORTB,3, when low, switches the unit off.
;*****************************************************************************

Suicide
	call	KeysOff
	clrf	PORTB
	goto	$

;*****************************************************************************
;* KeysOff
;* Loop until both keys off for 34 ms, then exit
;*  
;* Input variables: none
;* Output variables: TIMOUTH is cleared to 0
;*****************************************************************************

KeysOff
AnyOn
	clrf	TIMOUTH
BothOff
	btfsc	PORTA,2
	btfss	PORTA,3
	goto	AnyOn
	call	loop130		; 130us
	decfsz	TIMOUTH,F
	goto	BothOff
	return
;;*******
;*******  ANALYZER
;*******

;*****************************************************************************
;*  Pointer2
;*  Writes two measuring points for analyzer reference. First is TIMOUTL*10+5,
;*  then TIMOUTL is incremented by 2 and the second one is TIMOUTL*10+0
;*  
;*  Input variables: TIMOUTL will first be printed as TIMOUTL*10+5
;*  Output variables: TIMOUTL is incremented by 3 (pointer advanced by 30)
;*****************************************************************************

Pointer2
	call	Pointer1	; first two digits
	movlw	'5'		; third digit
	incf	TIMOUTL,F
	call	AdvToCh
	call	Pointer1	; first two digits
	movlw	'O'		; third digit
AdvToCh
	incf	TIMOUTL,F
	goto	Char

;*****************************************************************************
;*  Pointer1
;*  Writes blank, then symbol "^" and then converts TIMOUTL and prints
;*  as 2 digits.
;*  
;*  Input variables: TIMOUTL, bin value which will be printed as 2-digits
;*  Output variables: none
;*****************************************************************************

Pointer1
	call	Blank
	movlw	'^'		; "^" (pointing tool)
ChPtl
	call	Char
PrintTL
	movf	TIMOUTL,W
	goto	Print255	; first two digits

;*****************************************************************************
;* Mode1Show
;* Write measuring points at row 2, and wait until keys are released.
;*  Then increment SHOWCOU in range 0...4 and continue to Draw subroutine
;* This command, which executes when key 2 is pressed in analyzer mode,
;*  while the cursor is on the number of 60-sample group, advances the
;*  pointer.
;* It continues to Draw routine.
;*****************************************************************************

Mode1Show
	call	Row2
	bcf	STATUS,C
	rlf	SHOWCOU,W	; W=SHOWCOU*2
	movwf	TIMOUTL
	rlf	TIMOUTL,F	; TIMOUTL-SHOWCOU*4
	addwf	TIMOUTL,F	; TIMOUTL-SHOWCOU*6

	call	Pointer2	; print 1st and 2nd pointer
	call	Pointer2	; print 3rd and 4th pointer

	call	KeysOff		; wait until key off
	call	PrintM1		; restore normal row 2

	incf	SHOWCOU,F
	btfsc	SHOWCOU,2
	btfss	SHOWCOU,0
	goto	Draw
	clrf	SHOWCOU		; SHOWCOU cycle in range 0...4

;*****************************************************************************
;* Draw
;* This subroutine writes 20 pseudographic characters in line 1 in analyzer
;*  mode. First the whole buffer is rotated 0/60/120/180/240 bit places to
;*  the right (if SHOWCOU=0/1/2/3/4, respectively) to adjust the sequence
;*  to be displayed to the start of the buffer. Then the string of 20
;*  3-bit groups is rotated right, and displayed as 20 special characters
;*  (codes 0-7), defined at program setup (at loop labeled GoGraph). Then
;*  the buffer is rotated again, to the total of 304 bit places, so the
;*  buffer contents is unmodified on exit.
;*  
;* Input variables:
;*         SHOWCOU, denotes which group of 60 samples will be displayed
;* Output variables: none
;*****************************************************************************

Draw				; draw graphic in line 1
	movf	SHOWCOU,W
	btfss	STATUS,Z
	call	Carusel		; rotate buffer SHOWCOU*60 times

	call	Row1
	movlw	.20		; characters to write
	movwf	CHARCOU
Go20Chars
	clrf	SCRATCH
	call	RRBuf
	rlf	SCRATCH,F
	call	RRBuf
	rlf	SCRATCH,F
	call	RRBuf
	rlf	SCRATCH,W
	call	CharNCC		; draw one char = 3 samples
	decfsz	CHARCOU,F
	goto	Go20Chars

	movf	SHOWCOU,W
	sublw	.4		; W=4-SHOWCOU
	btfss	STATUS,Z
	call	Carusel		; rotate buffer again to restore it
	call	RRBuf4		; four more times to get 304 times
	goto	Farm1

;*****************************************************************************
;* Carusel
;* This subroutine rotates BUFFER right W*60 times
;*    Note: 0 times will be performed as 1024 times
;*  
;* Input variables: 
;*         W, how many (*60) times the buffer will be rotated right.
;* Output variables: none
;*****************************************************************************

Carusel
	movwf	SCRATCH		; SCRATCH=W
	swapf	SCRATCH,F	; SCRATCH=W*16
	subwf	SCRATCH,F	; SCRATCH=W*15
RRLoop
	call	RRBuf4		; 4 rotates here
	decfsz	SCRATCH,F
	goto	RRLoop		; rotate BUFFER right SCRATCH*4 times
	return

;*****************************************************************************
;* RRBuf4 executes RRBuf 4 times
;* RRBuf rotates buffer (38 bytes=304 bits) right for one bit position. Bit
;*  STATUS,C is first loaded from the first bit in buffer, so rotating will
;*  be completely performed at 304 bits, not through C.
;*  
;* Input variables: none
;* Output variables: none
;*****************************************************************************

RRBuf4				; 4 times rotates right 38 bytes
	call	RRBuf2
RRBuf2
	call	RRBuf
RRBuf				; rotates right 38 bytes, bit in C at exit
	movlw	BUFFER+.37
	movwf	FSR
	movlw	.38
	movwf	COUNT

	bcf	STATUS,C
	btfsc	BUFFER,0
	bsf	STATUS,C	; prepare to perform non-destructive rotating
ByteLoop
	rrf	INDF,F
	decf	FSR,F
	decfsz	COUNT,F
	goto	ByteLoop
	return			; bit in STATUS,C

;*****************************************************************************
;* Mode1Go
;* This is entry point for analyzer start (symbol "*"). After clearing
;*  line 1 of LCD, the program vectors to routines which handle different
;*  sampling rates. Three highest rates (1 MHz, 500 KHz and 228 KHz are
;*  sampled at individual routines (Get1MHz, Get500KHz and Get228KHz), and
;*  the remining rates at routine GetSlowClk. All those routines (except
;*  Get1MHz, which is location-sensitive (so it is at the very beginning of
;*  the program), are listed here.
;*  
;* Input variables: none
;* Output variables: none
;*****************************************************************************

Mode1Go				; *** ept: analyzer start

	call	ClrRow1		; clear LCD line 1 and turn LEDs off

	movf	RATE,W
	btfsc	STATUS,Z
	goto	Get1MHz		; if RATE=0

				; --- try 500 KHz rate
	movwf	SCRATCH
	decfsz	SCRATCH,F
	goto	Try228KHz	; if not RATE=1, then try Get228KHz

;*****************************************************************************
;* Get500KHz
;* This subroutine fetches 304 samples at 2us rate (5 instruction cycles
;*  timing). 
;* Call Common initializes loop counter (COUNT) to 38*8=304 samples
;*  and FSR to point to BUFFER. It also presets T0SE bit depended on
;*  XTOX bit (in FLAG register)to enable proper edge detecting, as it
;*  will affect TMR0 state.
;* State of key 2 (Break) is tested while waiting for starting condition.
;*  
;* Input variables: none
;* Output variables: none
;*****************************************************************************

Get500KHz			; 5 t read cycle
	movlw	.38
	call	Common
Edge500
	btfss	PORTA,3
	goto	Break

	movf	TMR0,W
	btfsc	STATUS,Z
	goto	Edge500
	decf	FSR,F
Get500Loop
	rrf	PORTA,F ; <--
	incf	FSR,F
	rrf	INDF,F
	movlw	.6
	movwf	DJNZ
Go6Bits
	rrf	PORTA,F ; <-- 6*
	rrf	INDF,F
 	decfsz	DJNZ,F
	goto	Go6Bits

	rrf	PORTA,F ; <--
	rrf	INDF,F
 	decfsz	COUNT,F
	goto	Get500Loop
	goto	Finished

;*****************************************************************************
;* Test if register SCRATCH reaches 0 after decrementing (this happens
;*  if RATE = 2), if so drop to Get228KHz else go to GetSlowClk
;*****************************************************************************

Try228KHz
	decfsz	SCRATCH,F
	goto	GetSlowClk	; if not RATE=2

;*****************************************************************************
;* Get228KHz
;* This subroutine fetches 304 samples at 4.4us rate (11 instruction
;*  cycles timing). 
;*  Call Common304 initializes loop counter (COUNT) to 304 samples
;*  and FSR to point to BUFFER. It also presets T0SE bit depended on
;*  XTOX bit (in FLAG register)to enable proper edge detecting, as it
;*  will affect TMR0 state.
;* State of key 2 (Break) is tested while waiting for starting condition.
;*  
;* Input variables: none
;* Output variables: none
;*****************************************************************************

Get228KHz			; 11 t read cycle
	call	Common304
Edge228
	btfss	PORTA,3
	goto	Break

	movf	TMR0,W
	btfsc	STATUS,Z
	goto	Edge228
Go228
	goto	$+1		; 2
Go228B
	rrf	PORTA,F	; <---	; 1
	rrf	INDF,F		; 1

	decf	COUNT,W		; 1
	andlw	7		; 1
	btfsc	STATUS,Z	; 1 (2)
	incf	FSR,F		; 1 (0)

	decfsz	COUNT,F		; 1
	goto	Go228		; 2	11T

	decfsz	TIMOUTH,F
	goto	Go228B		; this does not add extra cycles, as it
				; ...jumps after goto $+1
	goto	Finished

;*****************************************************************************
;* GetSlowClk
;* This subroutine fetches 304 samples at variable rate, depended on
;*  RATE (SCRATCH=RATE-3). Timing constant is loaded from lookup table
;*  located at DATA EEPROM (locations 30h-3ch).
;*  
;* Call Common304 initializes 16-bit loop counter to 304 samples
;*  (lo byte: COUNT=.304-.256, hi byte: TIMOUTH=.1)
;*  and FSR to point to BUFFER. It also presets T0SE bit depended on
;*  XTOX bit (in FLAG register)to enable proper edge detecting, as it
;*  will affect TMR0 state.
;*  
;* Rates 3-11 (100KHz-2.4KHz) have loop period of factor took from EEPROM
;*  table multiplied by 5 instruction cycles and adding 20 instruction
;*  cycles (Factor*5T+20T), and rates 12-15 (1.2KHz-200Hz) multilied the
;*  factor by 417 instruction cycles and adding 415 instruction cycle
;*  (Factor*417T+415T)
;*  
;*  RATE =  3, factor:   1, T/cycle:    25, Freq:  100   KHz
;*  RATE =  4, factor:   6, T/cycle:    50, Freq:   50   KHz
;*  RATE =  5, factor:   9, T/cycle:    65, Freq:   38.4 KHz
;*  RATE =  6, factor:  16, T/cycle:   100, Freq:   25   KHz
;*  RATE =  7, factor:  22, T/cycle:   130, Freq:   19.2 KHz
;*  RATE =  8, factor:  46, T/cycle:   250, Freq:   10   KHz
;*  RATE =  9, factor:  48, T/cycle:   260, Freq:    9.6 KHz
;*  RATE = 10, factor: 100, T/cycle:   520, Freq:    4.8 KHz
;*  RATE = 11, factor: 204, T/cycle:  1040, Freq:    2.4 KHz
;*  RATE = 12, factor:   5, T/cycle:  2500, Freq:    1   KHz
;*  RATE = 13, factor:  14, T/cycle:  6253, Freq:  400    Hz
;*  RATE = 14, factor:  59, T/cycle: 25018, Freq:  100    Hz
;*  RATE = 15, factor: 149, T/cycle: 62548, Freq:   40    Hz
;*  
;* State of key 2 (Break) is tested while waiting for starting condition.
;* LED P is turned on while sampling, to indicate sample period at slower
;*  rates, in which it appears to be visible.
;*  
;* Input variables: RATE, affects timing factor
;* Output variables: none
;*****************************************************************************

GetSlowClk
	decf	SCRATCH,W
	addlw	.48

	call	AGet_EE
	movwf	SCRATCH		; timing constant in SCRATCH
				; rates 3...15
	call	Common304	; lo byte loop counter = COUNT
EdgeSlow
	btfss	PORTA,3
	goto	Break

	movf	TMR0,W
	btfsc	STATUS,Z 
	goto	EdgeSlow

	bsf	PORTB,5		; led P on

GoSlow				;-----------------------------------------
	rrf	PORTA,F	; <---	; 1   this is input from PORTA,0
	rrf	INDF,F		; 1   store C to RAM BUFFER

	decf	COUNT,W		; 1
	andlw	7		; 1
	btfsc	STATUS,Z	; 1
	incf	FSR,F		; 1

	btfsc	RATE,3		; 1 1
	btfss	RATE,2		; 2 1
	goto	Not417		; - 2
				; ---------- 417 (RATE 12-15, bits 2,3 set)
	movf	SCRATCH,W	; 1
	movwf	CHARCOU		; 1
Loop417				;   \
	movlw	.82		; 1  \
	movwf	DJNZ		; 1   \
	call	Loop7		; 411  > 417*SCRATCH - 1
	nop			; 1   /
	decfsz	CHARCOU,F	; 1  / 2
	goto	Loop417		; 2 /  0

	movf	PORTB,W		; 1
	andlw	0dfh		; 1     reset bit 5 (LED P)
	btfss	COUNT,4		; 1 2
	iorlw	20h		; 1 0   set bit 5 (LED P) if COUNT,4 = 0
	movwf	PORTB		; 1     blink LED P

	movlw	.78		; 1
	goto	SameAs5		; 2
Not417				; ---------- 5 (RATE 3-11)
	goto	$+1		; 2
	movf	SCRATCH,W	; 1
SameAs5				; ---------- 417 and 5 (RATE 3-15)
	movwf	DJNZ		; 1
	call	GoLoop		; for 5: 3T+5T*SCRATCH, for 417: 393T

	decfsz	COUNT,F		; 1
	goto	GoSlow		; 2 --------------------------------------

	decfsz	TIMOUTH,F
	goto	GoSlow		; this adds 2T extra only once,after 48th pass
Finished
	clrf	SHOWCOU
	goto	Draw

;*****************************************************************************
;* Common304
;* This subroutine initializes low byte loop counter (COUNT) to 304 samples
;*  (lo byte: COUNT=.304-.256, hi byte: TIMOUTH=.1+1)
;*  and FSR to point to BUFFER. It also presets T0SE bit depended on
;*  XTOX bit (in FLAG register)to enable proper edge detecting, as it
;*  will affect TMR0 state.
;* Entry point Common allows subroutines Get1MHz and GetSlowClk to preset
;*  COUNT to another values.
;*  
;* Input variables:
;*    For entry point COMMON, register W is placed in COUNT
;* Output variables:
;*    COUNT is initialized to number of loop passes (W or low byte of 304)
;*    TMR0 is cleared
;*    T0SE and PORTB,2 are copied from FLAG,XTOX
;*****************************************************************************

Common304
	movlw	.2
	movwf	TIMOUTH		; hi byte loop counter for 304 passes
	movlw	.304-.256
Common
	movwf	COUNT
	movlw	BUFFER
	movwf	FSR

	movlw	28h		; lo2hi transition
	bcf	PORTB,2

	btfsc	FLAG,XTOX
	goto	ToOption

	movlw	38h		; hi2lo transition
	bsf	PORTB,2
ToOption
	bsf	STATUS,RP0
	movwf	OPTION_REG
	bcf	STATUS,RP0
	clrf	TMR0
	return

;*****************************************************************************
;* PrintM1
;* Print string at line 2 in analyzer mode.
;* At pos 0, rate in format XX[[.]X][M|K]Hz/XX[[.]X][u|m]s is printed. Those
;*  values are picked from table located in Data EEPROM, locations 0-2Fh.
;*  Register CHARCOU is used to fill blanks up to pos 13 in line 2, to
;*  disable phantom characters appearance when changing rate from some
;*  long-string to short-string value.
;* Symbol for starting or rising edge for starting event is written at pos
;*  13, symbol "*" for strt command at pos 15 and the number of group
;*  displayed at pos. 17.
;*  
;* Input variables:
;*    RATE will be printed in pos 0, row2
;*    bit FLAG,XTOX affects the printed symbol of rising/falling edge
;*    SHOWCOU (number of group displayed) is printed as numeric (+1)
;* Output variables: none
;*****************************************************************************

PrintM1
	call	Row2

	movlw	.13
	movwf	CHARCOU

	movf	RATE,W
	addwf	RATE,W
	addwf	RATE,W		; w = 3 * RATE

	call	AGet_EE
	movwf	TIMOUTL		; TIMOUTL = 1st byte

	swapf	TIMOUTL,W
	call	Get_EE

	btfsc	TIMOUTL,6
	bsf	FLAG,DP

	call	Print3		; rate freq display

	movlw	'M'		; MHz
	movf	RATE,F
	btfss	STATUS,Z
	movlw	'K'		; KHz
	btfsc	TIMOUTL,7
	call	Char
	movlw	TxtHz-1+1	; Hz
	call	Write
				; ----- here follows period disp
	movf	TIMOUTL,W

	call	Get_EE

	btfsc	TIMOUTL,2
	bsf	FLAG,DP

	call	Print3		; period display

	movlw	0e4h		; Greek micro
	btfss	TIMOUTL,3
	movlw	'm'		; m
	call	Char
	movlw	's'		; s
	call	Char
XtraChar			; adds extra (CHARCOU) blanks
	movlw	' '
	call	CharNCC
	decfsz	CHARCOU,F
	goto	XtraChar
				; ----- here follows starting condition symbol

	movlw	1		; symbol of rising edge
	btfss	FLAG,XTOX
	movlw	4		; symbol of falling edge
	call	CharBl
KaoAna				; ----- here follows Go symbol
	movlw	'*'		; *
	call	CharBl
				; ---- here follows number of 60 samples group
	incf	SHOWCOU,W
	call	Num
	call	Blank
Arrow
	movlw	7eh		; right arrow in LM032L character set
	goto	Char

;*****************************************************************************
;* AGet_EE
;* Get_EE
;*  This is routine for reading from Data EEPROM. Writing to BIN4+0 and
;*  BIN4+1 is also integrated here, as those variables are used for bin to
;*  decimal convertsion.
;*  
;* Input variables: W, data address at AGet_EE
;* Output variables: BIN4, binary data for rate display from DATA EEPROM
;*****************************************************************************

AGet_EE
	movwf	EEADR
Get_EE
	andlw	3
	movwf	BIN4+1		; hi byte BIN4 for freq display

	bsf	STATUS,RP0	; reading from eeprom
	bsf	EECON1,RD
	bcf	STATUS,RP0
	movf	EEDATA,W
	incf	EEADR,F
	movwf	BIN4		; lo byte BIN4 for freq display
	bcf	FLAG,DP
	return

;*****************************************************************************
;* ClrRow1
;* SameAs20
;* This subroutine clears line 1 of LCD and switches off LEDs. Entry point
;*  SameAs20 allows clearing some other number of character positions
;*  starting from line 1 of LCD. Alll LEDs are turned off, flags for LEDs
;*  also Cursor pointer of LCD is restored to first pos of line 1 at exit of
;*  subroutine.
;*  
;* Input variables:
;*          entry point ClrRow1:  none
;*          entry point SameAs20: W=number of characters to be cleared
;* Output variables:
;*          CHARCOU is decrementeted by the number of cleared characters
;*          SHOWCOU is cleared to 0
;*****************************************************************************
;* Row1
;*  This subroutine moves cursor to pos 0 of row 1 on LCD.
;*  
;*  Input variables: none
;*  Output variables: none
;*****************************************************************************

ClrRow1
	movlw	.20
SameAs20
	movwf	SHOWCOU	
	call	Row1
LoopClD
	call	Blank
	decfsz	SHOWCOU,F
	goto	LoopClD
 
	movlw	1fh
	andwf	FLAG,F		; turn LED flags off
Row1
	movlw	080h		; command for line 1
	goto	WrComL
;;*******
;*******  SERIAL CODE RECEIVER
;*******

;*****************************************************************************
;* Mode2Show
;* This subroutine advances SHOWCOU in range 0...5, and then continues to
;*  subroutine Show2
;*****************************************************************************

Mode2Show
	incf	SHOWCOU,F
	btfsc	SHOWCOU,2
	btfss	SHOWCOU,1
	goto	Show2
	clrf	SHOWCOU		; cycle show counter in range 0...5

;*****************************************************************************
;* Show2
;* This subroutine prints the 7 bytes of Buffer (+0, +7, +14, +21, +28 or
;*  +35) in hex mode at line 1, and the same bytes in ASCII at line 2. ASCII
;*  representation is with bit 7 reset, and non-printables (<20h) are printed
;*  as dots
;*  
;* Input variables:
;*     SHOWCOU, denotes which group of 7 bytes will be displayed
;* Output variables:
;*     CHARCOU is decremented by the number of characters printed
;*****************************************************************************

Show2
	movlw	BUFFER-1
	btfsc	SHOWCOU,2
	addlw	.28
	btfsc	SHOWCOU,1
	addlw	.14
	btfsc	SHOWCOU,0
	addlw	.7
	movwf	FSR		; FSR on (1st byte pos)-1 to display

	call	Row1
	movlw	.7		; bytes to display
	movwf	SCRATCH
Hex7
	incf	FSR,F
	swapf	INDF,W
	call	HexDigit
	movf	INDF,W
	call	HexDigit	; display hex digit
	call	Blank

	decfsz	SCRATCH,F
	goto	Hex7

	movf	FSR,W
	addlw	-.7
	movwf	FSR		; FSR on (1st byte pos)-1 to display

	call	Row2
	movlw	.7		; bytes to display
	movwf	SCRATCH
Ascii7
	incf	FSR,F
	movf	INDF,W
	andlw	7fh		; reduce ascii representation to 7 bits

	addlw	-20h
	addlw	20h
	btfsc	STATUS,C
	movlw	0a5h		; represent non-printables as dots

	call	Char		; display ascii char

	decfsz	SCRATCH,F
	goto	Ascii7

	goto	Farm2

;*****************************************************************************
;* Mode2Go
;* This is entry point for Start command in mode 2 (serial code receiver)
;*  Line 1 and first 7 positions (ASCII chars) of line 2 on LCD is cleared.
;*  Here the buffer is cleared and a sequence of 42 bytes are received and
;*  written to buffer. Manual break (key 2) jumps to Break handling point.
;* This protocol is used: High start bit, 7 or 8 data (true) bits, 0 or
;*  1 parity bit (not written to memory) and 1 low stop bit (not tested
;*  for validity). If RXBITS,2 is set then the input is inverted.
;* Baud rates 1200-115200 are supported.
;* Note: no receive errors are detected nor indicated.
;*  
;* Input variables:
;*  RXRATE (range 0...7), which affects timing loaded from table BaudRate
;*  RXBITS, bits 0-2 significant: bit0=parity, bit 1=7/8 bits, bit 2=inverse
;*  
;*  Output variables:
;*  Buffer (42 bytes) loaded with bytes received, all unreceived bytes
;*  represented as 00s.
;*****************************************************************************

Mode2Go
	bcf	PORTB,2
	btfsc	RXBITS,2
	bsf	PORTB,2
	call	ClrBuf
	movlw	.47
	call	SameAs20	; clear displayed HEX and ASCII values

	movlw	BUFFER
	movwf	FSR

	movf	RXRATE,W
	addlw	BaudRate
	call	PclSub		; rate in w!
RX42Bytes
	movwf	DJNZ		; baud rate factor is in W

	clrf	COUNT
	bsf	COUNT,3		; COUNT=8

	btfss	RXBITS,0
	btfss	RXBITS,1
	incf	COUNT,F		; if not(RXBITS and 3 = 2) then COUNT=9

	btfsc	RXBITS,0
	btfsc	RXBITS,1
	decf	COUNT,F		; if not(RXBITS and 3 = 1) reduce to 8 or 7

	btfss	RXBITS,2
	goto	GetSp2
				; inverse rx
GetSp1
	btfss	PORTA,4
	goto	GetSp1
GetStart1
	btfss	PORTA,3
	goto	BrkRS		; manual Break
	btfsc	PORTA,4
	goto	GetStart1
	goto	StartFound
				; true rx
GetSp2
	btfsc	PORTA,4
	goto	GetSp2
GetStart2
	btfss	PORTA,3
	goto	BrkRS		; manual Break
	btfss	PORTA,4
	goto	GetStart2
StartFound

				; 2-9 t from starting edge
HalfBit
	bsf	PORTB,5		; 1*W   led P on
	nop			; 1*W
	decfsz	DJNZ,F		; 1*W
	goto	HalfBit		; 2*W   0.5 bit loop

RX8Bits				;--------------------
	movwf	DJNZ		;  1
OneBit
	call	Waist8T		; 8 8 * W
	decfsz	DJNZ,F		; 1 2 * W
	goto	OneBit		; 2 - * W   one bit loop

	call	Waist6T		;  6

	rrf	PORTA,F		;  1       <---
	rrf	INDF,F		;  1

	decfsz	COUNT,F		;  1
	goto	RX8Bits		;  2 total 22t + (w-1) * 11
				;-------------------- received byte @ INDF
	btfsc	RXBITS,0
	rlf	INDF,F		; discard parity bit if received

	btfss	RXBITS,2
	comf	INDF,F

	bcf	STATUS,C
	btfsc	RXBITS,1
	rrf	INDF,F		; rotate 8th bit if 7 bits

	incf	FSR,F
	btfsc	FSR,6
	btfss	FSR,4
	goto	RX42Bytes	; if not yet FSR=50h
	goto	Show2
;;*******
;*******  FREQUENCY COUNTER
;*******

;*****************************************************************************
;* GoPresc
;* Prescaler factor (variable PRESC, in range 0...3) is advanced
;*  (executes when key 2 is pressed in frequency counter mode)
;*****************************************************************************

GoPresc
	incf	PRESC,F
	bcf	PRESC,2		; prescaler range 0...3

;*****************************************************************************
;* FreqEp
;* Frequency counter entry point.
;* Subroutine WrParam does this: Displays message "Frequency" in row 1,
;*  counter range (taken from table RangeTab) and resolution (taken from
;*  table PrescTab) in row2.
;* LEDs are turned OFF, and the main counter (BIN4, 4 bytes) cleared. The
;*  following is used to count pulses:
;* State of TMR0 is written to BIN4+0 and sequencialy tested, and when the
;*  bit 7 of current value detected to be 0 and the previous one was 1, the
;*  overflow is considerd. In that case, the state of BIN4+1 is advanced,
;*  extended to BIN4+2. After 500 ms, the 32- bit value of BIN4 is shifted
;*  left in a total of PRESC+2 times, to get multiplication by 4, 8, 16 or
;*  32. Then BIN+4 (4 bytes) is converted to ASCII and printed on LCD.
;* This routine does not call keyboard routine, as the accurate timing of
;*  500 ms must be generated for counting (high count register is CHARCOU,
;*  low count SHOWCOU). Instead of this, there is the individual routine for
;*  key 1 and key 2 test, and also the countdown timer for automatic power
;*  off (registers TIMOUTL, TIMOUTH).
;* If key 1 is  pressed, mode 1 (analyzer) is entered. If key 2 is
;*  pressed, prescaler value is advanced.
;*  
;* Note: code from label "Loop500A" to comment "; 500 ms timeout" is real
;*  time code. If the number of cycles is changed, then the literals
;*  0f4h+1 and 24h+1 written to CHARCOU and SHOWCOU must be readjusted.
;*  
;* Input variables: PRESC (affects prescaler factor)
;* Output variables: none
;*****************************************************************************


FreqEp				; mode 3: frequency counter
	call	WrParam
	call	KeysOff		; test both keys off for 34 ms and
				; initialize 8 min auto off sequence
Count500
	movlw	0d2h+REL
	call	WrComL		; cursor on right arrow 

	clrf	BIN4+1
	clrf	BIN4+2
	clrf	BIN4+3
	clrf	SCRATCH

	movlw	0f4h+1
	movwf	CHARCOU		; hi count byte
	movlw	24h+1		; 0f424h = .62500 cycles = 1250000 T = 500 ms
	movwf	SHOWCOU		; lo count byte

	movf	PRESC,W
	addlw	20h		; for PRESC 0,1,2,3   w = 20h,21h,22h,23h
	call	ToOption	; here is clrf TMR0 also
Loop500A
	btfss	PORTA,3		; 2
	goto	GoPresc		; -   prescaler change
Loop500
	movf	TMR0,W		; 1 1 1   the only place where TMR0 is read
	movwf	BIN4		; 1 1 1   rtcc ---> freq0

	rlf	BIN4,W		; 1 1 1   carry <--- TMR0.7
	rlf	SCRATCH,F	; 1 1 1   rotor <--- carry

	movf	SCRATCH,W	; 1 1 1
	andlw	3		; 1 1 1
	xorlw	2		; 1 1 1   00000000 if 1 <--- 0
	
	btfss	STATUS,Z	; 1 2 2 |     skip if TMR0 overflow
	goto	NotOvf1		; 2 - - |     if nz
				;	| 5T any case
	incf	BIN4+1,F	; - 1 1 |     nsb
NotOvf1	btfsc	STATUS,Z	; 2 2 1 |
	incf	BIN4+2,F	; - - 1 |     msb

	movlw	Head4-1		; 1
	btfss	PORTA,2		; 2
	goto	Mode4		; -   shortcut from mode 3 to mode 4

	decfsz	SHOWCOU,F	; 1 (2)   lo loop counter
	goto	Loop500A	; 2 (-)   20T total

				; ---------------------------
	decfsz	CHARCOU,F	;   (1)   hi loop counter	
	goto	Loop500		;   (2)   20T total

				; 500 ms timeout
	incf	PRESC,W
	movwf	CHARCOU
	incf	CHARCOU,F
ShLoop				; BIN4 = BIN4 * 2 total (PRESC+2) times
	bcf	STATUS,C
	rlf	BIN4, F
	rlf	BIN4+1,F
	rlf	BIN4+2,F
	rlf	BIN4+3,F
		
	decfsz	CHARCOU,F
	goto	ShLoop

	movlw	8ah
	call	WrComL		; cursor to freq display
	call	Print8		; print the frequency in 8-digit ASCII

	decfsz	TIMOUTL,F
	goto	Count500
	bsf	TIMOUTL,2	;TIMOUTL divide by 4, 4*256*500ms=512s=8.5min

	decfsz	TIMOUTH,F
	goto	Count500
				; 8 min timeout
	goto	Suicide

;;*******
;*******  BINARY TO ASCII CONVERSION
;*******

;*****************************************************************************
;* Headline
;* Clears SHOWCOU (Headline2 skips this), Clears LCD, prints right
;*  arrow @ last pos of row 2 and prints message addresed by W+1 at page 0.
;* Terminator is last character with bit 7 set.
;* 
;* Input variables: W+1 addresses string (on page 0) to be printed
;* Output variables:
;*    CHARCOU is decremented by the number of characters printed
;*****************************************************************************

Headline				; print 'analyzer','serial'...
	clrf	SHOWCOU
Headline2
	movwf	SCRATCH
	movlw	.59
	call	SameAs20
	goto	GoWrite

;*****************************************************************************
;* WrParam
;* Prints message "Frequency" in line 1 and parameters for frequency counter
;*  in line 2.
;* Text XXMHz/RYY is printed, where XX is taken from RangeTab, and YY from
;*  PrescTab.
;*  
;* Input variables:
;*    PRESC, affects displayed frequency range and resolution
;* Output variables:
;*    CHARCOU is decremented by the number of characters printed
;*****************************************************************************
;* Print255
;* Entry point Print255 converts 8-bit binary value (<100) in BIN4 to
;*  2-digit ASCII and prints it on LCD, without decimal point. Leading zeros
;*  are printed.
;*  
;* Input variables: W, binary number (0-99) to be converted and printed
;* Output variables:
;*    CHARCOU is decremented by the number of characters printed
;*****************************************************************************
;* Print3
;* Entry point Print3 converts 16-bit binary value (<1000) in BIN4 to 2- or
;*  3-digit ASCII and prints it on LCD. Leading zero is skipped only if value
;*  is <100. Decimal point is printed between tens and ones if FLAG,DP is
;*  set, otherwise decimal point is omited.
;*  
;* Input variables: BIN4 (2 bytes, LSB first, in range 0-999) binary number
;*                       to be converted and printed
;* Output variables:
;*    CHARCOU is decremented by the number of characters printed
;*****************************************************************************
;* PrintBR
;* Entry point PrintBR does the same as PRINT3, but the low byte value is
;*  in W instead in BIN4+0. This is used for baud rate display.
;*  
;* Input variables: W, BIN+1 (2 bytes, LSB in W, MSB in BIN+1, in range
;*  0-999) binary number to be converted and printed
;* Output variables:
;*    CHARCOU is decremented by the number of characters printed
;*****************************************************************************

WrParam				; print xxMHz/Rxx
	movlw	Head3-1
	call	Headline2
	call	Row2

	movlw	RangeTab
	call	Presc255

	movlw	TxtHz-1
	call	Write
	movlw	'R'
	call	Char

	movlw	PrescTab
Presc255
	addwf	PRESC,W
	call	PclSub
Print255
	bcf	FLAG,DP
	clrf	BIN4+1
PrintBR
	movwf	BIN4
Print3				; convert BIN4(16) and print 3 decimal digits
	clrf	BIN4+3
	clrf	BIN4+2
	clrf	CMP4+3
	clrf	CMP4+2
	clrf	CMP4+1
	movlw	.100
	call	Times

	btfss	STATUS,Z
	call	Num		; print if W>0

	movlw	.10
	call	Times
	call	Num		; print digit

	movlw	'.'
	btfsc	FLAG,DP
	call	Char		; print decimal point
	goto	NumBin4		; print digit

;*****************************************************************************
;* Times
;* Counts how many times CMP4 (32-bit value) "goes" in BIN4 (32-bit value).
;* BIN4 is sequentialy subtracted by CMP4 and counter COUNT advanced. When
;*  borrow is detected, BIN4 is restored to the last positiv value (by ADDing
;*  CMP4 again), COUNTer decremented and written to W.
;*  
;* Input variables: CMP4 (32-bit value), BIN4 (32-bit value)
;* Output variables:
;*    BIN4 (32-bit value) modified to mod(CMP4)
;*    W (in range 0...9) = BIN4 (32-bit value) / CMP4 (32-bit value)
;*****************************************************************************

Times
	movwf	CMP4
				; W = how many times CMP4 goes in BIN4 (32bit)
	clrf	COUNT
GoTD
	incf	COUNT,F
	call	Sub4		; BIN4=BIN4-CMP4  nc if result <0
	btfsc	STATUS,C
	goto	GoTD
	call	Add4		; BIN4=BIN4+CMP4  c set if ovf
	decf	COUNT,W
	return

;*****************************************************************************
;* Print8
;* This subroutine converts 32-bit value in BIN4 (low byte first), to
;*  8-digit ASCII and prints it on LCD. Leading zeros are printed as blanks.
;* Table DecTab (21 words, must be at page 0 if PCLATH=0) is used in conversion.
;*
;* Input variables: BIN4 (32-bit value, <.100,000,000)
;* Output variables:
;*    CHARCOU is decremented by the number of characters printed
;*****************************************************************************

Print8				; bin2dec conv BIN4(32), print 8 decimal digits
	movlw	DecTab-1	; inici tab ptr
	movwf	SCRATCH		; SCRATCH = tab ptr
	bcf	FLAG,RIPPLE	; zeros initialy printed as blanks, until
				; ...first non-zero appears
Cif7
	clrf	CMP4+3		; Clear CMP4+3, it is =0 in all cases
	call	PclSub2
	movwf	CMP4+2		; load decimal const from table in CMP4+2
	call	PclSub2
	movwf	CMP4+1		; load decimal const from table in CMP4+1
	call	PclSub2
	call	Times
	call	NZNum		; print if w>0 or RIPPLE=1, else blank

	movf	SCRATCH,W
	addlw	.237-DecTab
	btfss	STATUS,C
	goto	Cif7		; loop 7 x
NumBin4
	movf	BIN4,W
	goto	Num		; last digit, printed unconditionally

;*****************************************************************************
;* Sub4
;* Subtract CMP4 (32-byte value) from 32-bit value in BIN4 (low byte first)  
;* This is performed as adding of negative value of CMP4. Negating is
;*  performed as comlementing and incrementing by 1.
;* Note: Incrementing by 1 is performed on least significant byte only,
;*  without 32-bit extension, for code space saving. This will not cause
;*  error in this case, as the number of all possible values for CMP4+0 is
;*  limited and none of them is equal to 0FFh before incrementing
;*  (all possible values are taken from table DecTab, and are: 0ah, 64h,
;*  0e8h, 10h, 0a0h, 40h and 80h, and their negative values).
;*  However, this is valid if this subroutine is used for decimal
;*  conversion only, and if it is used for some other application,
;*  extension to 32-bit should be added after incrementing.
;*
;* Input variables: BIN4 (32-bit value), CMP4 (32-bit value)
;* Output variables:
;*    BIN4 (32-bit value)
;*    STATUS,C denotes the sign of result: if cleared, output value
;*             is negative (there is borrow)
;*  
;*  Note: Entry points Waist8T and Waist6T are used only by some real-time
;*  routines, in that case the instructions are dummy
;*****************************************************************************

Sub4				; 32-bit sub: BIN4 = BIN4 - CMP4
				; NC if result<0
	call	NegCmp
	call	Add4
NegCmp
	comf	CMP4+0,F
Waist8T
	comf	CMP4+1,F
	comf	CMP4+2,F
Waist6T
	comf	CMP4+3,F
	incf	CMP4+0,F	; neg = complement + 1
				; (no need to test overflow here, it will
				; ...never reach 0 after incrementing)
	return

;*****************************************************************************
;* Add4
;* Add CMP4 (32-byte value) to BIN4 (32-byte value). Four-instruction groups
;*  (movf-btfsc-incfsz-addwf) is used istead of non-existing ADD WITH CARRY.
;* Input variables: BIN4 (32-bit value), CMP4 (32-bit value)
;* Output variables: BIN4 (32-bit value), 33th bit in STATUS,C
;*****************************************************************************

Add4				; 32-bit add: BIN4 = BIN4 + CMP4
	movf	CMP4,W
	addwf	BIN4,F

	movf	CMP4+1,W
	btfsc	STATUS,C
	incfsz	CMP4+1,W	
	addwf	BIN4+1,F

	movf	CMP4+2,W
	btfsc	STATUS,C
	incfsz	CMP4+2,W
	addwf	BIN4+2,F

	movf	CMP4+3,W
	btfsc	STATUS,C
	incfsz	CMP4+3,W
	addwf	BIN4+3,F

	return

;*****************************************************************************
;* PclSub is used for indirect addressing
;* PclSub1 uses SCRATCH instead of W as input parameter
;* PclSub2 advances pointer SCRATCH before executong
;*  
;* Note: PCLATH=0 in all cases. So all tables pointed by this routine
;*  are on page 0
;*****************************************************************************

PclSub2
	incf	SCRATCH,F
PclSub1
	movf	SCRATCH,W
PclSub
	movwf	PCL		; jump to address pointed by PCLATH,W

;*****************************************************************************
;* ClrBuf
;* ClrRam
;* Subroutine ClrBuf clears BUFFER (42 bytes)
;* Entry point ClrRam allows some other start point for claring. It clears
;*  internal RAM from address in W to the location 7Fh.
;*  (locations 50h-7Fh, which do not exist in 16F84, are dummy).
;*
;* Both entry points continue to disabling Enable signal for LCD and
;*  33.8 ms timing loop
;*
;* Input variables:
;*   W is the start address if area to be cleared (ClrRam entry point only)
;* Output variables: none
;*****************************************************************************
  
ClrBuf
	movlw	BUFFER
ClrRam
	movwf	FSR
Zeros
	clrf	INDF
	incf	FSR,F
	btfss	FSR,7
	goto	Zeros

;*****************************************************************************
;*  Entry point DisEna30: Remove enable and discharge signal, and
;*                          refresh LEDs. Then loop 33.8 ms
;*  Entry point Wait30: Loop 33.8 ms
;*  Input variables: none
;*  Output variables: none
;*****************************************************************************

DisEna30
	call	DisEna
Wait30
	clrf	COUNT		; waist 33.8 ms
GoWait30
	call	loop130
	decfsz	COUNT,F
	goto	GoWait30
	return

;*****************************************************************************
;* NZNum
;* Num
;* Print numeric value in W (in range 0...9) on LCD. If FLAG,RIPPLE is
;*  cleared, 0 is printed as blank. If non-zero numeric is printed, it
;*  automatically sets FLAG,RIPPLE.
;* 0 (30h) is printed as capital O (4Fh), for improved readibility, as 0
;*  may easily be substituted by 8 on LCD. This changing 0 to O is not
;*  performed only in ASCII representation of recorded bytes in serial code
;*  receiver.
;* Entry point NUM prints numeric unconditionally, undependently of bit
;*  FLAG,RIPPLE.
;*
;* Input variables:
;*    W (0...9), number to be printed at current cursor position of LCD
;* Output variables:
;*    CHARCOU is decremented by the number of characters printed
;*****************************************************************************

NZNum				; same aso Num, only blank instead of 0
	btfsc	FLAG,RIPPLE
	goto	Num		; if RIPPLE set, no more blanks
	addlw	0
	btfsc	STATUS,Z
	goto	Blank
	bsf	FLAG,RIPPLE
Num				; numeric W to LCD, 0 change to capital O
	andlw	0fh
	btfsc	STATUS,Z
	movlw	'O'-30h
	addlw	30h
	goto	Char

;;*******
;*******  LCD ROUTINES
;*******

;*****************************************************************************
;*  All these entry ponts of this subroutine are used in program:
;*  
;*  Row2:    Issues command to move cursor to row 2 of LCD, and loops 130 us,
;*             to allow time for LCD controller to execute the command
;*  WrComl:  Issues command in W to the LCD, and loops 130 us, to allow time
;*             to LCD controller to execute the command
;*  loop130: Loops 130 us including call and return
;*  GoLoop:  Loops W*2 us
;*  Loop7:   Same as GoLoop, only 2t shorter (used in sample rate routine)
;*****************************************************************************

Row2
	movlw	0c0h		; command for line 2

WrComL				; issues command in W
	call	WrCom
loop130				; * waist 130 us
	clrf	DJNZ
	bsf	DJNZ,6		; DJNZ = 40h = .64
GoLoop
	goto	$+1		; 2
Loop7
	decfsz	DJNZ,F		; 1
	goto	GoLoop		; 2    64x5=320t (128 us)
	return

;*****************************************************************************
;*  All these entry ponts of this subroutine are used in program:
;*  
;*  CharBl:  print character in W and then blank on LCD and advance CHARCOU
;*  Blank:   print blank (32h) on LCD and decrement CHARCOU
;*  Char:    print character in W on LCD and decrement CHARCOU
;*  CharNCC: print character in W on LCD without affecting CHARCOU
;*  
;*  Note: CHARCOU is used to print fixed format message on LCD, as the
;*  calling routine will add N-CHARCOU blanks to fill area N characters long
;*  
;*  Input variables:
;*         all entry points except Blank: W = character to be printed
;*  Output variables:
;*         all entry points except CharNCC: CHARCOU is decremented by one
;*****************************************************************************

CharBl
	call	Char
Blank				; print blank
	movlw	' '
Char				; print w
	decf	CHARCOU,F
CharNCC
	bsf	PORTB,1		; rs hi (data)
	goto	Skr1

;*****************************************************************************
;* PrintBrk
;*  Print message "Break" in row 1, pos 0
;*  
;* Input variables: none
;* Output variables: 
;*    CHARCOU is decremented by the number of characters printed
;*****************************************************************************
;* Write
;* Print message addressed by W+1 in line 1
;* Note: Terminator is last character in string with bit 7 set
;*  
;* Input variables: W points to string (RETLWs) decremented by 1
;* Output variables: 
;*     CHARCOU is decremented by the number of characters printed
;*****************************************************************************

PrintBrk
	call	Row1
	movlw	BrkMes-1

Write				; write string at (SCRATCH+1), terminator 0
	movwf	SCRATCH
GoWrite
	call	PclSub2
	addlw	80h
	btfsc	STATUS,C
	goto	Char		; last character was with bit 7 set
	andlw	7fh		; restore initial character value
	call	Char
	goto	GoWrite

;*****************************************************************************
;* Wrcom
;*  Write command in W to LCD, then loop 130 us
;* Skr1
;*  Allows data write to LCD, if PORTB,1 is set previously
;* Nibble
;*  Write one nibble (W,0-3) to LCD data bus
;*  
;* Note: all entry ponits are terminated by 130us timing loop, to allow
;*  LCD controller to execute accepted command/data.
;*
;* Input variables: command in W
;* Output variables: none
;*****************************************************************************

WrCom
	bcf	PORTB,1		; rs lo (command)
Skr1
	movwf	DJNZ
	call	Hinib_B		; outputs w,4-7 to PORTB,4-7
	call	EnaLCD
Nibble
	swapf	DJNZ,W
	call	Hinib_B		; outputs w,0-3 to PORTB,4-7
	call	EnaLCD
	goto	loop130

;*****************************************************************************
;* Generate Enable signal (1200us) for LCD controller, and refresh LEDs.
;* Entry point DisEna: Remove enable and discharge signal, and refresh LEDs.
;*****************************************************************************

EnaLCD
	bsf	PORTB,0		; enable lcd
	goto	$+1
DisEna
	bcf	PORTB,0

;*****************************************************************************
;* MoveLEDs
;*  Entry point MoveLEDs: transfer from FLAG,LEDP FLAG,LEDH and FLAG,LEDL to
;*                        PORTB,5 PORTB,6 and PORTB,7 to service LEDs.
;*
;* Input variables: 
;*    FLAG, bits LEDP, LEDH, LEDH will affect LED1, LED2, LED3 respectively
;* Output variables: none
;*****************************************************************************
;* Hinib_B
;*  Entry point Hinib_B:  output W,4-7 to 4-bit LCD data bus
;*  
;* Input variables: 
;*    hi nibble of W is copied to LCD data bus
;* Output variables: none
;*****************************************************************************

MoveLESd				; puts states for LEDs L,H,P at PORTB,5-7
	movf	FLAG,W
Hinib_B				; outputs w,4-7 to PORTB,4-7
	bcf	PORTB,4
	bcf	PORTB,5
	bcf	PORTB,6
	bcf	PORTB,7
	andlw	0f0h
	iorwf	PORTB,F
	return

;*****************************************************************************
;* HexDigit
;* This subroutine prints low nibble of W on LCD as hexadecimal digit. Zero
;*  (30h) is printed as capital O (7Fh)
;*  
;* Input variables: W in range 0...0fh, hex number to be printed
;* Output variables: CHARCOU is decremented by two
;*****************************************************************************

HexDigit			; hex W (lo nibble) to LCD, change 0...
				; ...to capital O
	andlw	0fh
	movwf	BIN4
	addlw	-0ah
	btfss	STATUS,C
	goto	NumBin4
	addlw	.7+3ah		; 3ah to 'A' correction
	goto	Char

;;*******
;*******  DATA EEPROM
;*******

;*****************************************************************************
;*  This table is located in data eeprom
;*  It contains numerical data for 16 sample frequencies period display
;*  for analyzer. Last 13 bytes are timing constants used by subroutine
;*  GetSlowClk to generate internal timings (three fastest rates need no
;*  constants from the table, as they are treated as special cases)
;*****************************************************************************
;*  ----- TABLE 1 (00h-2Fh): Rate display table for analyzer
;*  
;*  **** First byte: Flags. Bits in this byte have the following functions:
;*  bit 7 = 0: Frequency in Hz
;*        = 1: Frequency in Mhz or Khz
;*  bit 6 = 0: Frequency does not contain decimal point
;*        = 1: Frequency contains decimal point before last digit
;*  bits  5,4: Bits 9 and 8 for frequency, respectively
;*  bit 3 = 0: Period in us (microseconds)
;*        = 1: Period in ms (miliseconds)
;*  bit 2 = 0: Period does not contain decimal point
;*        = 1: Period contains decimal point before last digit
;*  bits  1,0: Bits 9 and 8 for period, respectively
;*  **** Second byte: low significant byte for frequency
;*  **** Third byte: low significant byte for period
;*****************************************************************************
;*  ----- TABLE 2 (30h-3Ch): Timing constant table for analyzer
;*  
;*  Timing constant factors for all sample rates generated by subroutine
;*  GetSlowClk (all except 1MHz, 500KHz and 228KHz)
;*****************************************************************************
;*  Note: This is read-only data, so the Data EEPROM must be programmed
;*  before the unit is ready to use. MCU will affect data EEPROM contents.
;*  If your programmer does not support loading of Data EEPROM contents, it
;*  must be loaded manualy. This will help in that case (all values are
;*  hexadecimal):
;*  
;*  addr 00-07:  88 01 01 98 F4 02 8C E4
;*  addr 08-0f:  2C 88 64 0A 88 32 14 D8
;*  addr 10-17:  80 1A 88 19 28 C8 C0 34
;*  addr 18-1f:  88 0A 64 C8 60 68 C8 30
;*  addr 20-27:  D0 C9 18 A1 80 01 01 14
;*  addr 28-2f:  90 19 00 64 0A 00 28 19
;*  addr 30-37:  01 06 09 10 16 2E 30 64
;*  addr 38-3c:  CC 05 0E 3B 95
;*  
;*  Total bytes used: 61
;*****************************************************************************

	org	2100h

 	;	  K.hiu.hi  (sufix KHz, dp, hi bits, sufix us, dp, hi bits)
	;
	;			       SCRATCH t/cycle	Hz	s     RATE

;	dt	b'10001000', .1,   .1	; -  	2.5	1M	1u	0
;	dt	b'10011000', .244, .2	; -  	5	500K	2u	1
;	dt	b'10001100', .228, .44	; -  	11	228K	4.4u	2
;	dt	b'10001000', .100, .10	; 1	25	100K   	10u	3
;	dt	b'10001000', .50,  .20	; 6	50	50K    	20u	4
;	dt	b'11011000', .128, .26	; 9	65	38.4K  	26u	5
;	dt	b'10001000', .25,  .40	; 16	100	25K	40u	6
;	dt	b'11001000', .192, .52	; 22	130	19.2K  	52u	7
;	dt	b'10001000', .10,  .100	; 46	250	10K	100u	8
;	dt	b'11001000', .96,  .104	; 48	260	9.6K   	104u	9
;	dt	b'11001000', .48,  .208	; 100	520	4.8K   	208u	10
;	dt	b'11001001', .24,  .161	; 204	1040	2.4K   	417u	11
;	dt	b'10000000', .1,   .1	; 5	2500	1K	1m	12
;	dt	b'00010100', .144, .25	; 14	6253	400	2.5m	13
;	dt	b'00000000', .100, .10	; 59	25018	100	10m	14
;	dt	b'00000000', .40,  .25	; 149	62548	40	25m	15

	;	rate table

;	dt	.001, .006, .009, .016, .022, .046, .048
;	dt	.100, .204, .005, .014, .059, .149

	end
